function report_absence(reporting) {
  "use strict";
  var model = openerp;

  reporting.ReportAbsenceWidget = reporting.Base.extend({
    template: 'ReportAbsence',
    content: [],
    rowsData: [],
    modules: ['hr'],
    events: {
      'click #toolbar > button': 'clickOnAction',
      'click #generate': 'fetchGenerate',
      'click-row.bs.table #table ': 'clickAnalysisDetail',
      'change #current-date': 'ShowDateRange',
      'change #current-store': 'updateEmployee',
      'change #current-company': 'updateStore',
      'change #current-period': 'updatePeriodSelections',
    },

    init: function(parent) {
      this._super(parent);
    },

    start: function() {
      var date = new model.eiru_reports.ReportDatePickerWidget(self);
      date.fecthFecha();
      this.fetchInitial();
    },

    checkModel: function(model) {
      var self = this;
      return _.filter(self.modules, function(item) {
        return item.name === model
      });
    },

    valorNull: function(dato) {
      var valor = "";
      if (dato) {
        valor = dato;
      }
      return valor;
    },

    ShowDateRange: function() {
      var self = this;
      var date = self.$el.find('#current-date').val();
      if (date == 'range') {
        self.$el.find('.datepicker').css('display', 'block');
      }
      if (date != 'range') {
        self.$el.find('.datepicker').css('display', 'none');
      }
    },

    clickAnalysisDetail: function(e, row, $element, field) {
      if (field == 'employee_name') {
        this.do_action({
          name: "Registro de Funcionario",
          type: 'ir.actions.act_window',
          res_model: "hr.employee",
          views: [
            [false, 'form']
          ],
          target: 'new',
          domain: [
            ['id', '=', row.id]
          ],
          context: {},
          flags: {
            'form': {
              'action_buttons': false,
              'options': {
                'mode': 'view'
              }
            }
          },
          res_id: row.id,
        });
      }
      if (field == 'payslip') {
        this.do_action({
          name: "Nómina",
          type: 'ir.actions.act_window',
          res_model: "hr.payslip",
          views: [
            [false, 'form']
          ],
          target: 'new',
          domain: [
            ['id', '=', row.payslip_id]
          ],
          context: {},
          flags: {
            'form': {
              'action_buttons': false,
              'options': {
                'mode': 'view'
              }
            }
          },
          res_id: row.payslip_id,
        });
      }
      e.stopImmediatePropagation();
    },

    fetchInitial: function() {
      var self = this;
      self.fecthIrModuleModule().then(function(IrModuleModule) {
        return IrModuleModule;
      }).then(function(IrModuleModule) {
        self.IrModuleModule = IrModuleModule;
        return self.fetchResCompany();
      }).then(function(ResCompany) {
        self.ResCompany = ResCompany;
        if (ResCompany.length > 1) {
          self.$el.find('#current-company').append('<option value="9999999">Todas las empresas</option>');
          _.each(ResCompany, function(item) {
            self.$el.find('#current-company').append('<option value="' + item.id + '">' + item.name + '</option>');
          });
        } else {
          self.$el.find('.company').css('display', 'none');
        }
        return self.fetchResStore();
      }).then(function(ResStore) {
        self.ResStore = ResStore;
        if (ResStore.length > 0) {
          self.$el.find('#current-store').append('<option value="9999999">Todas las sucursales</option>');
          _.each(ResStore, function(item) {
            self.$el.find('#current-store').append('<option value="' + item.id + '">' + item.name + '</option>');
          });
        } else {
          self.$el.find('.store').css('display', 'none');
        }
        return self.fetchHrEmployee();
      }).then(function(HrEmployee) {
        self.HrEmployee = HrEmployee;
        if (HrEmployee.length > 1) {
          self.$el.find('#current-employee').append('<option value="9999999">Todos los funcionarios</option>');
          _.each(HrEmployee, function(item) {
            self.$el.find('#current-employee').append('<option value="' + item.id + '">' + item.name + '</option>');
          });
        } else {
          self.$el.find('.employee').css('display', 'none');
        }
        return self.fetchAccountPeriod();
      }).then(function(AccountPeriod) {
        self.AccountPeriod = AccountPeriod;
        if (AccountPeriod.length > 1) {
          self.$el.find('#current-period').append('<option value="9999999">Todos los períodos</option>');
          _.each(AccountPeriod, function(item) {
            self.$el.find('#current-period').append('<option value="' + item.id + '">' + item.name + '</option>');
          });
        } else {
          self.$el.find('.period').css('display', 'none');
        }
        return self.fetchResCurrency();
      }).then(function(ResCurrency) {
        self.ResCurrency = ResCurrency;
      });
      self.$el.find('#generate').css('display', 'inline');
      return;
    },

    fetchGenerate: function() {
      var self = this;
      self.$el.find('.search-form').block({
        message: null,
        overlayCSS: {
          backgroundColor: '#FAFAFA'
        }
      });
      self.$el.find('.report-form').block({
        message: null,
        overlayCSS: {
          backgroundColor: '#FAFAFA'
        }
      });

      this.fetchHrPayslip().then(function(HrPayslip) {
        return HrPayslip;
      }).then(function(HrPayslip) {
        self.HrPayslip = HrPayslip;
        return self.fetchHrPayslipLine();
      }).then(function(HrPayslipLine) {
        self.HrPayslipLine = HrPayslipLine;
        return self.fetchHrPayslipWorkedDays();
      }).then(function(HrPayslipWorkedDays) {
        self.HrPayslipWorkedDays = HrPayslipWorkedDays;
        return self.BuildTable();
      });
    },

    fecthIrModuleModule: function() {
      var self = this;
      var defer = $.Deferred();
      var fields = ['name', 'id'];
      var domain = [
        ['state', '=', 'installed'],
        ['name', 'in', self.modules]
      ];
      var IrModuleModule = new model.web.Model('ir.module.module');
      IrModuleModule.query(fields).filter(domain).all().then(function(results) {
        defer.resolve(results);
      })
      return defer;
    },

    fetchResCompany: function() {
      var self = this;
      var defer = $.Deferred();
      var company = new model.web.Model('res.company');
      var field = ['id', 'name', 'currency_id', 'logo'];
      company.query(field).filter().all().then(function(results) {
        defer.resolve(results);
      });
      return defer;
    },

    getResCompany: function(id) {
      var self = this;
      return _.filter(self.ResCompany, function(item) {
        return item.id == id;
      })
    },

    fetchResStore: function() {
      var self = this;
      var defer = $.Deferred();
      var store = new model.web.Model('res.store');
      var field = ['id', 'name', 'company_id'];
      store.query(field).filter().all().then(function(results) {
        defer.resolve(results);
      });
      return defer;
    },

    fetchResCurrency: function() {
      var self = this;
      var defer = $.Deferred();
      var fields = ['id', 'name', 'symbol', 'rate_silent', 'base', 'decimal_separator', 'decimal_places', 'thousands_separator', 'symbol_position'];
      var domain = [
        ['active', '=', true]
      ];
      var ResCurrency = new model.web.Model('res.currency');
      ResCurrency.query(fields).filter(domain).all().then(function(results) {
        defer.resolve(results);
      });
      return defer;
    },

    getResCurrency: function(id) {
      var self = this;
      return _.filter(self.ResCurrency, function(item) {
        return item.id === id;
      })
    },

    fetchHrEmployee: function() {
      var self = this;
      var defer = $.Deferred();
      var employee = new model.web.Model('hr.employee');
      var field = ['id', 'name', 'store_id'];
      employee.query(field).filter().all().then(function(results) {
        defer.resolve(results);
      });
      return defer;
    },

    fetchAccountPeriod: function() {
      var self = this;
      var defer = $.Deferred();
      var period = new model.web.Model('account.period');
      var field = ['id', 'name','date_stop'];
      period.query(field).filter().all().then(function(results) {
        results.sort(function (a, b) {
          if (a.date_stop > b.date_stop) {
            return -1;
          }
          if (a.date_stop < b.date_stop) {
            return 1;
          }
          return 0;
        });
        defer.resolve(results);
      });

      return defer;
    },

    fetchHrPayslipLine: function() {
      var self = this;
      var defer = $.Deferred();
      var payslip_ids = _.flatten(_.map(self.HrPayslip, function(item) {
        return item.id;
      }));
      var domain = [
        ['slip_id', 'in', payslip_ids]
      ];

      var line = new model.web.Model('hr.payslip.line');
      var field = ['id', 'name', 'code', 'total'];
      line.query(field).filter(domain).all().then(function(results) {
        defer.resolve(results);
      });
      return defer;
    },

    fetchHrPayslip: function() {
      var self = this;
      var defer = $.Deferred();
      var company = self.$el.find('#current-company').val();
      var store = self.$el.find('#current-store').val();
      var employee_id = self.$el.find('#current-employee').val();
      var period = self.$el.find('#current-period').val();

      var domain = [];

      if (company && company != 9999999) {
        domain.push(['company_id', '=', parseInt(company)]);
      }

      if (store && store != 9999999) {
        domain.push(['employee_id.store_id', '=', parseInt(store)]);
      }

      if (employee_id && employee_id != 9999999) {
        domain.push(['employee_id', '=', parseInt(employee_id)]);
      }

      if (period && period != 9999999) {
        domain.push(['period_id', '=', parseInt(period)]);
      }

      var fields = [
        'id',
        'name',
        'company_id',
        'employee_id',
        'worked_days_line_ids',
        'line_ids'
      ];
      var HrPayslip = new model.web.Model('hr.payslip');
      HrPayslip.query(fields).filter(domain).all().then(function(results) {
        defer.resolve(results);
      });
      return defer;
    },

    fetchHrPayslipWorkedDays: function() {
      var self = this;
      var defer = $.Deferred();
      var date = self.$el.find('#current-date').val();
      var desde = self.$el.find('#from').val();
      var hasta = self.$el.find('#to').val();
      var payslip_ids = _.flatten(_.map(self.HrPayslip, function(item) {
        return item.id;
      }));

      var domain = [
        ['code', '=', 'FALTAS'],
        ['payslip_id', 'in', payslip_ids]
      ];

      if (date && date != 9999999) {
        if (desde) {
          var date = desde.split('/')
          date = (date[2] + "-" + date[1] + "-" + date[0]);
          domain.push(['create_date', '>=', date]);
        }
        if (hasta) {
          var date = hasta.split('/')
          date = (date[2] + "-" + date[1] + "-" + date[0]);
          domain.push(['create_date', '<=', date]);
        }
        if (date == 'today') {
          var today = moment().format('YYYY-MM-DD ');
          domain.push(['create_date', 'like', today]);
        }
        if (date == 'yesterday') {
          var yesterday = moment().add(-1, 'days').format('YYYY-MM-DD ');
          domain.push(['create_date', 'like', yesterday]);
        }
        if (date == 'currentMonth') {
          var currentMonth = moment().format('YYYY-MM');
          domain.push(['create_date', 'like', currentMonth]);
        }
        if (date == 'lastMonth') {
          var lastMonth = moment().add(-1, 'months').format('YYYY-MM');
          domain.push(['create_date', 'like', lastMonth]);
        }
      }

      var fields = [
        'id',
        'code',
        'name',
        'payslip_id',
        'create_date',
        'number_of_days',
      ];

      var HrPayslipWorkedDays = new model.web.Model('hr.payslip.worked_days');
      HrPayslipWorkedDays.query(fields).filter(domain).all().then(function(results) {
        defer.resolve(results);
      });
      return defer;
    },

    getHrPayslipWorkedDays: function(ids) {
      var self = this;
      return _.filter(self.HrPayslipWorkedDays, function(a) {
        return _.find(ids, function(b) {
          return b === a.id;
        });
      });
    },

    getHrPayslip: function(id) {
      var self = this;
      return _.filter(self.HrPayslip, function(a) {
          return a.employee_id[0] === id;
      });
    },

    getAmount: function(line_ids, code) {
      var self = this;
      var amount = 0;
      amount = _.map(_.filter(self.HrPayslipLine, function(a) {
        return _.find(line_ids, function(b) {
          return b === a.id && a.code == code;
        });
      }), function(map) {
        return map.total;
      });

      if (amount.length == 0) {
        return 0
      } else {
        return amount[0];
      }
    },

    updateEmployee: function() {
      var self = this;
      var store = self.$el.find('#current-store').val();
      if (store != 9999999) {
        var employee = self.$el.find('#current-employee').empty();
        self.$el.find('#current-employee').append('<option value="9999999">Todos los funcionarios</option>');
        _.each(self.HrEmployee, function(item) {
          if (parseFloat(store) == item.store_id[0]) {
            self.$el.find('#current-employee').append('<option value="' + item.id + '">' + item.name + '</option>');
          }
        });
      } else {
        var employee = self.$el.find('#current-employee').empty();
        self.$el.find('#current-employee').append('<option value="9999999">Todos los funcionarios</option>');
        _.each(self.HrEmployee, function(item) {
          self.$el.find('#current-employee').append('<option value="' + item.id + '">' + item.name + '</option>');
        });
      }
    },

    updateStore: function() {
      var self = this;
      var company = self.$el.find('#current-company').val();
      if (company != 9999999) {
        var store = self.$el.find('#current-store').empty();
        self.$el.find('#current-store').append('<option value="9999999">Todas las sucursales</option>');
        _.each(self.ResStore, function(item) {
          if (parseFloat(company) == item.company_id[0]) {
            self.$el.find('#current-store').append('<option value="' + item.id + '">' + item.name + '</option>');
          }
        });
      } else {
        var store = self.$el.find('#current-store').empty();
        self.$el.find('#current-store').append('<option value="9999999">Todas las sucursales</option>');
        _.each(self.ResStore, function(item) {
          self.$el.find('#current-store').append('<option value="' + item.id + '">' + item.name + '</option>');
        });
      }
    },

    updatePeriodSelections: function() {
      var self = this;
      var period = self.$el.find('#current-period').val();
      if (period != 9999999) {
        self.$el.find('#current-date').val(9999999);
        self.$el.find('#current-date').prop('disabled', 'disabled');
        self.$el.find('.datepicker').css('display', 'none');
      } else {
        self.$el.find('#current-date').prop('disabled', false);
      }
    },

    BuildTable: function() {
      var self = this;
      var data = [];
      var info = [];
      var company = $('#current-company').val();
      if (company && company != 9999999) {
        var ResCompany = self.getResCompany(company).shift();
        var CurrencyBase = self.getResCurrency(ResCompany.currency_id[0]).shift();
      } else {
        var CurrencyBase = self.getResCurrency(self.ResCompany[0].currency_id[0]).shift();
      }

      var HrEmployee = self.HrEmployee;
      _.each(HrEmployee, function(each){
        var total_amount = 0;
        var total_days = 0;
        info = [];

          var HrPayslip = self.getHrPayslip(each.id);
          _.each(HrPayslip, function(item) {

            var HrPayslipWorkedDays = self.getHrPayslipWorkedDays(item.worked_days_line_ids);

            _.each(HrPayslipWorkedDays, function(index) {
              total_days = total_days + index.number_of_days;
              info.push({
                id: index.id,
                payslip_id : index.payslip_id[0],

                create_date: index.create_date,
                name: index.name,
                payslip : index.payslip_id[1],
                number_of_days: index.number_of_days,
              });
            });
            var amount = self.getAmount(item.line_ids, "FALTAS");
            total_amount = total_amount + amount;
          });

          if (total_amount != 0) {

            data.push({
              id: each.id,
              employee_name: each.name,

              total_days : accounting.formatNumber(total_days, '', CurrencyBase.decimal_places, CurrencyBase.thousands_separator, CurrencyBase.decimal_separator),
              total_amount: accounting.formatMoney(total_amount, '', CurrencyBase.decimal_places, CurrencyBase.thousands_separator, CurrencyBase.decimal_separator),

              total_days_no_format : total_days,
              amount_no_format : total_amount,

              decimal_places: CurrencyBase.decimal_places,
              thousands_separator: CurrencyBase.thousands_separator,
              decimal_separator: CurrencyBase.decimal_separator,

              info: info,

            });
          }
      })

      self.content = data;
      self.loadTable(data);
      self.$el.find('.report-form').css('display', 'block');
      self.$el.find('.search-form').unblock();
      self.$el.find('.report-form').unblock();
    },

    loadTable: function(rowsTable) {
      var self = this;
      var data = [];
      self.rowsData = rowsTable;
      var table = this.$el.find('#table');

      table.bootstrapTable({
        data:rowsTable,
        detailView: true,
        onExpandRow: function(index, row, $detail) {
          $detail.html('<div class="panel panel-first-style"><div class="panel-heading panel-header-first-style">Detalle de Ausencias</div><table id="table2"></table></div>').find('table').bootstrapTable({
            columns: [{
                      field: 'create_date',
                      title: 'Fecha',
                      align: 'center',
                  },
                  {
                      field: 'name',
                      title: 'Descripción',
                      align: 'left',
                  },
                  {
                      field: 'payslip',
                      title: 'Nómina',
                      align: 'left',
                      class: 'hover',
                  },
                  {
                      field: 'number_of_days',
                      title: 'Días Faltados',
                      align: 'center',
                  }
                ],

          data:row.info,
          })
            $('#table2').removeClass('table-hover').addClass('table-no-bordered');
            $('thead').css('background','none');
        }
      })

      table.bootstrapTable('load', rowsTable);
    },

    clickOnAction: function(e) {
      var self = this;
      var action = self.$el.find(e.target).val();
      var company = $('#current-company').val();
      if (company && company != 9999999) {
        var ResCompany = self.getResCompany(company).shift();

      } else {
        var ResCompany = self.ResCompany[0];
      }

      var table = self.$el.find("#table");
      var row = table.bootstrapTable('getData');

      if (action === 'pdf') {

        var pdf_title = 'Histórico de Ausencias';
        var pdf_type = '';
        var pdf_name = 'historico_de_ausencias_';
        var pdf_columnStyles = {
          create_date: {
            columnWidth: 30,
            halign: 'center'
          },

          name: {
            columnWidth: 50,
            halign: 'left'
          },

          payslip: {
            halign: 'left'
          },

          number_of_days: {
            columnWidth: 20,
            halign: 'center'
          },
        };

        var filter = self.getFilter();
        var pdf = new reporting.ReportPdfWidget(self);
        pdf.drawAbsencePdf(
          row,
          ResCompany,
          pdf_title,
          pdf_type,
          pdf_name,
          pdf_columnStyles,
          filter,
        );
      }
    },
    getFilter: function() {
      var self = this;
      var company = self.$el.find('#current-company').val();
      var store = self.$el.find('#current-store').val();
      var employee = self.$el.find('#current-employee').val();
      var period = self.$el.find('#current-period').val();
      var date = self.$el.find('#current-date').val();
      var desde = self.$el.find('#from').val();
      var hasta = self.$el.find('#to').val();

      var filter = [];

      if (company && company != 9999999) {
        var ResCompany = _.filter(self.ResCompany, function(item) {
          return item.id == company;
        });
        filter.push({
          title: 'Empresa',
          value: ResCompany[0].name,
        });
      }

      if (store && store != 9999999) {
        var ResStore = _.filter(self.ResStore, function(item) {
          return item.id == store;
        });

        filter.push({
          title: 'Sucursal',
          value: ResStore[0].name,
        });
      }

      if (employee && employee != 9999999) {
        var HrEmployee = _.filter(self.HrEmployee, function(item) {
          return item.id == employee;
        });

        filter.push({
          title: 'Funcionario',
          value: HrEmployee[0].name,
        });
      }

      if (period && period != 9999999) {
        var AccountPeriod = _.filter(self.AccountPeriod, function(item) {
          return item.id == period;
        })
        filter.push({
          title: 'Periodo',
          value: AccountPeriod[0].name,
        });
      }

      if (date && date != 9999999) {
        moment.locale('es', {
          months: 'Enero_Febrero_Marzo_Abril_Mayo_Junio_Julio_Agosto_Septiembre_Octubre_Noviembre_Diciembre'.split('_'),
        });

        if (date == 'range') {
          filter.push({
            title: 'Fecha',
            value: desde + ' al ' + hasta,
          });

        } else {
          if (date == 'today') {
            var fecha = moment().format('DD/MM/YYYY');
          }

          if (date == 'yesterday') {
            var fecha = moment().add(-1, 'days').format('DD/MM/YYYY');
          }
          if (date == 'currentMonth') {
            var fecha = moment().format('MMMM/YYYY');
          }
          if (date == 'lastMonth') {
            var fecha = moment().add(-1, 'months').format('MMMM/YYYY');
          }

          filter.push({
            title: 'Fecha',
            value: fecha,
          });
        }
      }
      return filter;
    },
  });
}
