function report_employee_list(reporting) {
  "use strict";

  var model = openerp;

  reporting.ReportEmployeeListWidget = reporting.Base.extend({
    template: 'ReportEmployeeList',
    AccountVoucher: [],
    content: [],
    rowsData: [],
    modules: ['hr'],
    events: {
      'click #toolbar > button': 'clickOnAction',
      'click #generate': 'fetchGenerate',
      'click-row.bs.table #table ': 'clickAnalysisDetail',
      'change #current-company': 'updateStore',
    },
    init: function(parent) {
      this._super(parent);
    },
    start: function() {
      var table = this.$el.find('#table');
      table.bootstrapTable({
        data: self.rowsData
      });
      var date = new model.eiru_reports.ReportDatePickerWidget(self);
      date.fecthFecha();
      this.fetchInitial();
    },

    checkModel: function(model) {
      var self = this;
      return _.filter(self.modules, function(item) {
        return item.name === model
      });
    },

    valorNull: function(dato) {
      var valor = "";
      if (dato) {
        valor = dato;
      }
      return valor;
    },

    clickAnalysisDetail: function(e, row, $element, field) {
      if (field == 'name') {
        this.do_action({
          name: "Funcionario",
          type: 'ir.actions.act_window',
          res_model: "hr.employee",
          views: [
            [false, 'form']
          ],
          target: 'new',
          domain: [
            ['id', '=', row.id]
          ],
          context: {},
          flags: {
            'form': {
              'action_buttons': false,
              'options': {
                'mode': 'view'
              }
            }
          },
          res_id: row.id,
        });
      }

      e.stopImmediatePropagation();
    },

    fetchInitial: function() {
      var self = this;
      self.fecthIrModuleModule().then(function(IrModuleModule) {
        return IrModuleModule;
      }).then(function(IrModuleModule) {
        self.IrModuleModule = IrModuleModule;
        return self.fetchResCompany();
      }).then(function(ResCompany) {
        self.ResCompany = ResCompany;
        if (ResCompany.length > 1) {
          self.$el.find('#current-company').append('<option value="9999999">Todas las empresas</option>');
          _.each(ResCompany, function(item) {
            self.$el.find('#current-company').append('<option value="' + item.id + '">' + item.name + '</option>');
          });
        } else {
          self.$el.find('.company').css('display', 'none');
        }
        return self.fetchResStore();
      }).then(function(ResStore) {
        self.ResStore = ResStore;
        if (ResStore.length > 1) {
          self.$el.find('#current-store').append('<option value="9999999">Todas las empresas</option>');
          _.each(ResStore, function(item) {
            self.$el.find('#current-store').append('<option value="' + item.id + '">' + item.name + '</option>');
          });
        } else {
          self.$el.find('.store').css('display', 'none');
        }
      });
      self.$el.find('#generate').css('display', 'inline');
      return;
    },

    fetchGenerate: function() {
      var self = this;
      self.$el.find('.search-form').block({
        message: null,
        overlayCSS: {
          backgroundColor: '#FAFAFA'
        }
      });
      self.$el.find('.report-form').block({
        message: null,
        overlayCSS: {
          backgroundColor: '#FAFAFA'
        }
      });

      this.fetchHrEmployee().then(function(HrEmployee) {
        return HrEmployee;
      }).then(function(HrEmployee) {
        self.HrEmployee = HrEmployee;
        return self.BuildTable();
      });
    },

    fecthIrModuleModule: function() {
      var self = this;
      var defer = $.Deferred();
      var fields = ['name', 'id'];
      var domain = [
        ['state', '=', 'installed'],
        ['name', 'in', self.modules],
      ];
      var IrModuleModule = new model.web.Model('ir.module.module');

      IrModuleModule.query(fields).filter(domain).all().then(function(results) {
        defer.resolve(results);
      });
      return defer;
    },

    /*====================================================================
        RES COMPANY
    ====================================================================*/
    fetchResCompany: function() {
      var self = this;
      var defer = $.Deferred();
      var currency = new model.web.Model('res.company');
      var field = ['id', 'name', 'currency_id', 'logo'];
      currency.query(field).filter().all().then(function(results) {
        defer.resolve(results);
      });
      return defer;
    },

    getResCompany: function(id) {
      var self = this;
      return _.filter(self.ResCompany, function(item) {
        return item.id == id;
      })
    },

    fetchResStore: function() {
      var self = this;
      var defer = $.Deferred();
      var store = new model.web.Model('res.store');
      var field = ['id', 'name','company_id'];
      store.query(field).filter().all().then(function(results) {
        defer.resolve(results);
      });
      return defer;
    },

    fetchHrEmployee: function() {
      var self = this;
      var defer = $.Deferred();
      var company = self.$el.find('#current-company').val();
      var store = self.$el.find('#current-store').val();
      var state = self.$el.find('#current-state').val();

      var domain = [
        '|', ['active', '=', true],
        ['active', '=', false]
      ];

      if (company && company != 9999999) {
        domain.push(['company_id', '=', parseInt(company)]);
      }

      if (store && store != 9999999) {
        domain.push(['store_id', '=', parseInt(store)]);
      }

      if (state && state != 9999999) {
        if (state == 'activo') {
          domain.push(['active', '=', true]);
        } else {
          domain.push(['active', '=', false]);
        }
      }

      var fields = [
        'id',
        'company_id',
        'name',
        'identification_id',
        'work_email',
        'work_phone',
        'job_id',
        'contracts_count',
        'leaves_count',
        'payslip_count',

      ];

      var HrEmployee = new model.web.Model('hr.employee');
      HrEmployee.query(fields).filter(domain).all().then(function(results) {
        defer.resolve(results);
      });
      return defer;
    },

    updateStore: function() {
      var self = this;
      var company = self.$el.find('#current-company').val();
      if (company != 9999999) {
        var store = self.$el.find('#current-store').empty();
        self.$el.find('#current-store').append('<option value="9999999">Todas las sucursales</option>');
        _.each(self.ResStore, function(item) {
          if (parseFloat(company) == item.company_id[0]) {
            self.$el.find('#current-store').append('<option value="' + item.id + '">' + item.name + '</option>');
          }
        });
      } else {
        var store = self.$el.find('#current-store').empty();
        self.$el.find('#current-store').append('<option value="9999999">Todas las sucursales</option>');
        _.each(self.ResStore, function(item) {
          self.$el.find('#current-store').append('<option value="' + item.id + '">' + item.name + '</option>');
        });
      }
    },

    /*====================================================================
        BUILD
    ====================================================================*/
    BuildTable: function() {
      var self = this;
      var data = [];

      var HrEmployee = self.HrEmployee;
      _.each(HrEmployee, function(item) {
        data.push({
          id: item.id,
          name: self.valorNull(item.name),
          company_id: self.valorNull(item.company_id[1]),
          identification_id: self.valorNull(item.identification_id),
          work_email: self.valorNull(item.work_email),
          work_phone: self.valorNull(item.work_phone),
          job_id: self.valorNull(item.job_id[1]),
          contracts_count: item.contracts_count,
          payslip_count: item.payslip_count,
        });
      });
      self.content = data;
      self.loadTable(data);
      self.$el.find('.report-form').css('display', 'block');
      self.$el.find('.search-form').unblock();
      self.$el.find('.report-form').unblock();
    },

    loadTable: function(rowsTable) {
      var self = this;
      self.rowsData = rowsTable;
      var table = this.$el.find('#table');
      table.bootstrapTable('load', rowsTable);
    },

    clickOnAction: function(e) {
      var self = this;
      var ResCompany;
      var action = this.$el.find(e.target).val();
      var company = $('#current-company').val();
      if (company && company != 9999999) {
        var ResCompany = self.getResCompany(company).shift();
      } else {
        var ResCompany = self.ResCompany[0];
      }
      var getColumns = [];
      var rows = [];
      var table = this.$el.find("#table");
      var column = table.bootstrapTable('getVisibleColumns');
      var row = table.bootstrapTable('getData');

      row.push({});
      if (action === 'pdf') {
        var data = _.map(column, function(val) {
          return val.field
        });
        _.each(_.map(column, function(val) {
          return val
        }), function(item) {
          getColumns.push([{
            title: item.title,
            dataKey: item.field
          }]);
        });
        var pdf_title = 'Lista de Funcionarios';
        var pdf_type = '';
        var pdf_name = 'lista_de_funcionarios_';
        var pdf_columnStyles = {
          identification_id: {
            columnWidth: 15,
            halign: 'center'
          },
          name: {
            halign: 'left'
          },
          job_id: {
            columnWidth: 20,
            halign: 'center'
          },
          work_email: {
            columnWidth: 50,
            halign: 'left'
          },
          work_phone: {
            columnWidth: 18,
            halign: 'center'
          },
          contracts_count: {
            columnWidth: 19,
            halign: 'center'
          },
          payslip_count: {
            columnWidth: 19,
            halign: 'center'
          },
        };
        var filter = self.getFilter();
        var pdf = new model.eiru_reports.ReportPdfWidget(self);
        pdf.drawPDF(
          _.flatten(getColumns),
          row,
          ResCompany,
          pdf_title,
          pdf_type,
          pdf_name,
          pdf_columnStyles,
          filter,
        );
      }
    },

    getFilter: function() {
      var self = this;
      var company = self.$el.find('#current-company').val();
      var store = self.$el.find('#current-store').val();
      var state = self.$el.find('#current-state').val();

      var filter = [];

      if (company && company != 9999999) {
        var ResCompany = _.filter(self.ResCompany, function(item) {
          return item.id == company;
        });
        filter.push({
          title: 'Empresa',
          value: ResCompany[0].name,
        });
      }

      if (store && store != 9999999) {
        var ResStore = _.filter(self.ResStore, function(item) {
          return item.id == store;
        });

        filter.push({
          title: 'Sucursal',
          value: ResStore[0].name,
        });
      }

      if (state && state != 9999999) {
        filter.push({
          title: 'Estado',
          value: $('#current-state option:selected').text(),
        });
      }
      return filter;
    },
  });
}
