function report_payslip(reporting) {
  "use strict";

  var model = openerp;

  reporting.ReportPayslipWidget = reporting.Base.extend({
    template: 'ReportPayslip',
    content: [],
    rowsData: [],
    modules: ['hr'],
    events: {
      'click #toolbar > button': 'clickOnAction',
      'click #generate': 'fetchGenerate',
      'click-row.bs.table #table ': 'clickAnalysisDetail',
      'change #current-store': 'updateEmployee',
      'change #current-company': 'updateStore',
    },

    init: function(parent) {
      this._super(parent);
    },

    start: function() {
      var table = this.$el.find('#table');
      table.bootstrapTable({
        data: self.rowsData
      });
      var date = new model.eiru_reports.ReportDatePickerWidget(self);
      date.fecthFecha();
      this.fetchInitial();
    },

    checkModel: function(model) {
      var self = this;
      return _.filter(self.modules, function(item) {
        return item.name === model
      });
    },

    valorNull: function(dato) {
      var valor = "";
      if (dato) {
        valor = dato;
      }
      return valor;
    },

    clickAnalysisDetail: function(e, row, $element, field) {
      if (field == 'employee_name') {
        this.do_action({
          name: "Funcionario",
          type: 'ir.actions.act_window',
          res_model: "hr.employee",
          views: [
            [false, 'form']
          ],
          target: 'new',
          domain: [
            ['id', '=', row.employee_id]
          ],
          context: {},
          flags: {
            'form': {
              'action_buttons': false,
              'options': {
                'mode': 'view'
              }
            }
          },
          res_id: row.employee_id,
        });
      }
      if (field == 'name') {
        this.do_action({
          name: "Nómina",
          type: 'ir.actions.act_window',
          res_model: "hr.payslip",
          views: [
            [false, 'form']
          ],
          target: 'new',
          domain: [
            ['id', '=', row.id]
          ],
          context: {},
          flags: {
            'form': {
              'action_buttons': false,
              'options': {
                'mode': 'view'
              }
            }
          },
          res_id: row.id,
        });
      }

      if (field == 'contract_name') {
        this.do_action({
          name: "Contrato",
          type: 'ir.actions.act_window',
          res_model: "hr.contract",
          views: [
            [false, 'form']
          ],
          target: 'new',
          domain: [
            ['id', '=', row.contract_id]
          ],
          context: {},
          flags: {
            'form': {
              'action_buttons': false,
              'options': {
                'mode': 'view'
              }
            }
          },
          res_id: row.contract_id,
        });
      }

      e.stopImmediatePropagation();
    },

    fetchInitial: function() {
      var self = this;
      self.fecthIrModuleModule().then(function(IrModuleModule) {
        return IrModuleModule;
      }).then(function(IrModuleModule) {
        self.IrModuleModule = IrModuleModule;
        return self.fetchResCompany();
      }).then(function(ResCompany) {
        self.ResCompany = ResCompany;
        if (ResCompany.length > 1) {
          self.$el.find('#current-company').append('<option value="9999999">Todas las empresas</option>');
          _.each(ResCompany, function(item) {
            self.$el.find('#current-company').append('<option value="' + item.id + '">' + item.name + '</option>');
          });
        } else {
          self.$el.find('.company').css('display', 'none');
        }
        return self.fetchResStore();
      }).then(function(ResStore) {
        self.ResStore = ResStore;
        if (ResStore.length > 1) {
          self.$el.find('#current-store').append('<option value="9999999">Todas las sucursales</option>');
          _.each(ResStore, function(item) {
            self.$el.find('#current-store').append('<option value="' + item.id + '">' + item.name + '</option>');
          });
        } else {
          self.$el.find('.store').css('display', 'none');
        }
        return self.fetchHrEmployee();
      }).then(function(HrEmployee) {
        self.HrEmployee = HrEmployee;
        if (HrEmployee.length > 1) {
          self.$el.find('#current-employee').append('<option value="9999999">Todos los funcionarios</option>');
          _.each(HrEmployee, function(item) {
            self.$el.find('#current-employee').append('<option value="' + item.id + '">' + item.name + '</option>');
          });
        } else {
          self.$el.find('.employee').css('display', 'none');
        }
        return self.fetchHrPayrollStructure();
      }).then(function(HrPayrollStructure) {
        self.HrPayrollStructure = HrPayrollStructure;
        if (HrPayrollStructure.length > 1) {
          self.$el.find('#current-structure').append('<option value="9999999">Todos los tipos</option>');
          _.each(HrPayrollStructure, function(item) {
            self.$el.find('#current-structure').append('<option value="' + item.id + '">' + item.name + '</option>');
          });
        } else {
          self.$el.find('.structure').css('display', 'none');
        }
        return self.fetchAccountPeriod();
      }).then(function(AccountPeriod){
        self.AccountPeriod = AccountPeriod;
        if (AccountPeriod.length > 1) {
          self.$el.find('#current-period').append('<option value="9999999">Todos los períodos</option>');
          _.each(AccountPeriod, function(item) {
            self.$el.find('#current-period').append('<option value="' + item.id + '">' + item.name + '</option>');
          });
        } else {
          self.$el.find('.period').css('display', 'none');
        }
        return self.fetchHrPayslipLine();
      }).then(function(HrPayslipLine) {
        self.HrPayslipLine = HrPayslipLine;
        return self.fetchResCurrency();
      }).then(function(ResCurrency) {
        self.ResCurrency = ResCurrency;
      });
      self.$el.find('#generate').css('display', 'inline');
      return;
    },

    fetchGenerate: function() {
      var self = this;
      self.$el.find('.search-form').block({
        message: null,
        overlayCSS: {
          backgroundColor: '#FAFAFA'
        }
      });
      self.$el.find('.report-form').block({
        message: null,
        overlayCSS: {
          backgroundColor: '#FAFAFA'
        }
      });

      this.fetchHrPayslip().then(function(HrPayslip) {
        return HrPayslip;
      }).then(function(HrPayslip) {
        self.HrPayslip = HrPayslip;
        return self.BuildTable();
      });
    },

    fecthIrModuleModule: function() {
      var self = this;
      var defer = $.Deferred();
      var fields = ['name', 'id'];
      var domain = [
        ['state', '=', 'installed'],
        ['name', 'in', self.modules]
      ];
      var IrModuleModule = new model.web.Model('ir.module.module');
      IrModuleModule.query(fields).filter(domain).all().then(function(results) {
        defer.resolve(results);
      })
      return defer;
    },

    fetchResCompany: function() {
      var self = this;
      var defer = $.Deferred();
      var currency = new model.web.Model('res.company');
      var field = ['id', 'name', 'currency_id', 'logo'];
      currency.query(field).filter().all().then(function(results) {
        defer.resolve(results);
      });
      return defer;
    },

    getResCompany: function(id) {
      var self = this;
      return _.filter(self.ResCompany, function(item) {
        return item.id == id;
      })
    },

    fetchResStore: function() {
      var self = this;
      var defer = $.Deferred();
      var store = new model.web.Model('res.store');
      var field = ['id', 'name', 'company_id'];
      store.query(field).filter().all().then(function(results) {
        defer.resolve(results);
      });
      return defer;
    },

    fetchResCurrency: function() {
      var self = this;
      var defer = $.Deferred();
      var fields = ['id', 'name', 'symbol', 'rate_silent', 'base', 'decimal_separator', 'decimal_places', 'thousands_separator', 'symbol_position'];
      var domain = [
        ['active', '=', true]
      ];
      var ResCurrency = new model.web.Model('res.currency');
      ResCurrency.query(fields).filter(domain).all().then(function(results) {
        defer.resolve(results);
      });
      return defer;
    },

    getResCurrency: function(id) {
      var self = this;
      return _.filter(self.ResCurrency, function(item) {
        return item.id === id;
      })
    },

    fetchHrEmployee: function() {
      var self = this;
      var defer = $.Deferred();
      var employee = new model.web.Model('hr.employee');
      var field = ['id', 'name','store_id'];
      employee.query(field).filter().all().then(function(results) {
        defer.resolve(results);
      });
      return defer;
    },

    fetchHrPayrollStructure: function() {
      var self = this;
      var defer = $.Deferred();
      var structure = new model.web.Model('hr.payroll.structure');
      var field = ['id', 'name'];
      structure.query(field).filter().all().then(function(results) {
        defer.resolve(results);
      });
      return defer;
    },

    fetchHrPayslipLine: function() {
      var self = this;
      var defer = $.Deferred();
      var line = new model.web.Model('hr.payslip.line');
      var field = ['id', 'name', 'code', 'total'];
      line.query(field).filter().all().then(function(results) {
        defer.resolve(results);
      });
      return defer;
    },

    fetchAccountPeriod: function() {
      var self = this;
      var defer = $.Deferred();
      var period = new model.web.Model('account.period');
      var field = ['id', 'name','date_stop'];
      period.query(field).filter().all().then(function(results) {
        results.sort(function (a, b) {
          if (a.date_stop > b.date_stop) {
            return -1;
          }
          if (a.date_stop < b.date_stop) {
            return 1;
          }
          return 0;
        });
        defer.resolve(results);
      });

      return defer;
    },

    fetchHrPayslip: function() {
      var self = this;
      var defer = $.Deferred();
      var company = self.$el.find('#current-company').val();
      var store = self.$el.find('#current-store').val();
      var state = self.$el.find('#current-state').val();
      var employee_id = self.$el.find('#current-employee').val();
      var struct_id = self.$el.find('#current-structure').val();
      var period = self.$el.find('#current-period').val();
      var domain = [];

      if (company && company != 9999999) {
        domain.push(['company_id', '=', parseInt(company)]);
      }

      if (store && store != 9999999) {
        domain.push(['employee_id.store_id', '=', parseInt(store)]);
      }

      if (state && state != 9999999) {
        domain.push(['state', '=', state]);
      }

      if (employee_id && employee_id != 9999999) {
        domain.push(['employee_id', '=', parseInt(employee_id)]);
      }

      if (struct_id && struct_id != 9999999) {
        domain.push(['struct_id', '=', parseInt(struct_id)]);
      }

      if (period && period != 9999999){
        domain.push(['period_id','=', parseInt(period)]);
      }

      var fields = [
        'id',
        'employee_id',
        'name',
        'number',
        'contract_id',
        'struct_id',
        'state',
        'line_ids',
        'period_id'
      ];
      var HrPayslip = new model.web.Model('hr.payslip');
      HrPayslip.query(fields).filter(domain).all().then(function(results) {
        defer.resolve(results);
      });
      return defer;
    },

    getAmount: function(line_ids, code) {
      var self = this;
      var amount = 0;
      amount =  _.map(_.filter(self.HrPayslipLine, function(a) {
        return _.find(line_ids, function(b) {
          return b === a.id && a.code == code;
        });
      }), function(map) {
        return map.total;
      });

      if(amount.length == 0){
        return 0
      }
      else {
        return amount[0];
      }
    },

    updateEmployee: function() {
      var self = this;
      var store = self.$el.find('#current-store').val();
      if (store != 9999999) {
        var employee = self.$el.find('#current-employee').empty();
        self.$el.find('#current-employee').append('<option value="9999999">Todos los funcionarios</option>');
        _.each(self.HrEmployee, function(item) {
          if (parseFloat(store) == item.store_id[0]) {
            self.$el.find('#current-employee').append('<option value="' + item.id + '">' + item.name + '</option>');
          }
        });
      } else {
        var employee = self.$el.find('#current-employee').empty();
        self.$el.find('#current-employee').append('<option value="9999999">Todos los funcionarios</option>');
        _.each(self.HrEmployee, function(item) {
          self.$el.find('#current-employee').append('<option value="' + item.id + '">' + item.name + '</option>');
        });
      }
    },

    updateStore: function() {
      var self = this;
      var company = self.$el.find('#current-company').val();
      if (company != 9999999) {
        var store = self.$el.find('#current-store').empty();
        self.$el.find('#current-store').append('<option value="9999999">Todas las sucursales</option>');
        _.each(self.ResStore, function(item) {
          if (parseFloat(company) == item.company_id[0]) {
            self.$el.find('#current-store').append('<option value="' + item.id + '">' + item.name + '</option>');
          }
        });
      } else {
        var store = self.$el.find('#current-store').empty();
        self.$el.find('#current-store').append('<option value="9999999">Todas las sucursales</option>');
        _.each(self.ResStore, function(item) {
          self.$el.find('#current-store').append('<option value="' + item.id + '">' + item.name + '</option>');
        });
      }
    },

    BuildTable: function() {
      var self = this;
      var data = [];
      var company = $('#current-company').val();
      if (company && company != 9999999) {
        var ResCompany = self.getResCompany(company).shift();
        var CurrencyBase = self.getResCurrency(ResCompany.currency_id[0]).shift();
      } else {
        var CurrencyBase = self.getResCurrency(self.ResCompany[0].currency_id[0]).shift();
      }

      var HrPayslip = self.HrPayslip;
      _.each(HrPayslip, function(item) {

        if (item.state == 'done') {
          var state = 'Hecho';
        }
        if (item.state == 'cancel') {
          var state = 'Cancelado';
        }
        if (item.state == 'paid') {
          var state = 'Pagado';
        }

        var salary = self.getAmount(item.line_ids, "BASIC");
        var ips = self.getAmount(item.line_ids, "IPSE");
        var cash_advance = self.getAmount(item.line_ids, "ADPER");
        var bonus = self.getAmount(item.line_ids, "CMS");
        var faltas = self.getAmount(item.line_ids, "FALTAS");
        var neto = self.getAmount(item.line_ids,"NET");

        data.push({
          id: item.id,
          contract_id : self.valorNull(item.contract_id[0]),
          employee_id: self.valorNull(item.employee_id[0]),

          employee_name: self.valorNull(item.employee_id[1]),
          number: self.valorNull(item.number),
          name: self.valorNull(item.name),
          contract_name: self.valorNull(item.contract_id[1]),
          struct_id: self.valorNull(item.struct_id[1]),
          salary: accounting.formatMoney(salary, '', CurrencyBase.decimal_places, CurrencyBase.thousands_separator, CurrencyBase.decimal_separator),
          ips: accounting.formatMoney(ips, '', CurrencyBase.decimal_places, CurrencyBase.thousands_separator, CurrencyBase.decimal_separator),
          faltas: accounting.formatMoney(faltas, '', CurrencyBase.decimal_places, CurrencyBase.thousands_separator, CurrencyBase.decimal_separator),
          cash_advance: accounting.formatMoney(cash_advance, '', CurrencyBase.decimal_places, CurrencyBase.thousands_separator, CurrencyBase.decimal_separator),
          bonus: accounting.formatMoney(bonus, '', CurrencyBase.decimal_places, CurrencyBase.thousands_separator, CurrencyBase.decimal_separator),
          neto: accounting.formatMoney(salary+bonus-faltas, '', CurrencyBase.decimal_places, CurrencyBase.thousands_separator, CurrencyBase.decimal_separator),
          totsalary: accounting.formatMoney((salary+cash_advance), '', CurrencyBase.decimal_places, CurrencyBase.thousands_separator, CurrencyBase.decimal_separator),
          state: state,
          period_id: item.period_id,

          salary_no_format: salary,
          totsalary_no_format: (salary+cash_advance),
          ips_no_format: ips,
          cash_advance_no_format: cash_advance,
          bonus_no_format: bonus,
          neto_no_format: salary+bonus-faltas,
          faltas_no_format: faltas,

          decimal_places : CurrencyBase.decimal_places,
          thousands_separator: CurrencyBase.thousands_separator,
          decimal_separator: CurrencyBase.decimal_separator,

        });
      });
      data.sort(function (a, b) {
        if (a.period_id > b.period_id) {
          return -1;
        }
        if (a.period_id < b.period_id) {
          return 1;
        }
        return 0;
      });
      self.content = data;
      self.loadTable(data);
      self.$el.find('.report-form').css('display', 'block');
      self.$el.find('.search-form').unblock();
      self.$el.find('.report-form').unblock();
    },

    loadTable: function(rowsTable) {
      var self = this;
      self.rowsData = rowsTable;
      var table = this.$el.find('#table');
      table.bootstrapTable('load', rowsTable);
    },

    clickOnAction: function(e) {
      var self = this;
      var action = self.$el.find(e.target).val();
      var company = $('#current-company').val();
      if (company && company != 9999999) {
        var ResCompany = self.getResCompany(company).shift();

      } else {
        var ResCompany = self.ResCompany[0];
      }
      var getColumns = [];
      var rows = [];
      var table = self.$el.find("#table");
      var column = table.bootstrapTable('getVisibleColumns');
      var row = table.bootstrapTable('getData');

      var salary = salaryFormatter(row);
      var ips = ipsFormatter(row);
      var cash_advance = cash_advanceFormatter(row);
      var bonus = bonusFormatter(row);
      var faltas = faltasFormatter(row);
      var neto = netoFormatter(row);
      var totsalary = totsalaryFormatter(row);

      row.push({
        employee_name: 'Totales',
        salary : salary,
        ips : ips,
        cash_advance : cash_advance,
        bonus : bonus,
        faltas : faltas,
        neto : neto,
        totsalary : totsalary,
      });

      if (action === 'pdf') {
        var data = _.map(column, function(val) {
          return val.field
        });
        _.each(_.map(column, function(val) {
          return val
        }), function(item) {
          getColumns.push([{
            title: item.title,
            dataKey: item.field
          }]);
        });
        var pdf_title = 'Análisis de Nóminas';
        var pdf_type = 'l';
        var pdf_name = 'analisis_de_nominas_';
        var pdf_columnStyles = {
          employee_name: {
            columnWidth: 25,
            halign: 'left'
          },
          number: {
            columnWidth: 14,
            halign: 'center'
          },
          name: {
            halign: 'left'
          },
          contract_name: {
            columnWidth: 30,
            halign: 'left'
          },
          struct_id: {
            columnWidth: 20,
            halign: 'center'
          },
          salary: {
            columnWidth: 23,
            halign: 'center'
          },
          ips: {
            columnWidth: 22,
            halign: 'center'
          },
          cash_advance: {
            columnWidth: 22,
            halign: 'center'
          },
          bonus: {
            columnWidth: 22,
            halign: 'center'
          },
          faltas: {
            columnWidth: 20,
            halign: 'center'
          },
          neto: {
            columnWidth: 25,
            halign: 'center'
          },
          totsalary: {
            columnWidth: 25,
            halign: 'center'
          },
          state: {
            columnWidth: 13,
            halign: 'center'
          },
        };

        var filter = self.getFilter();
        var pdf = new model.eiru_reports.ReportPdfWidget(self);
        pdf.drawPDF(
          _.flatten(getColumns),
          row,
          ResCompany,
          pdf_title,
          pdf_type,
          pdf_name,
          pdf_columnStyles,
          filter,
        );
      }
    },
    getFilter: function() {
      var self = this;
      var company = self.$el.find('#current-company').val();
      var store = self.$el.find('#current-store').val();
      var employee = self.$el.find('#current-employee').val();
      var structure = self.$el.find('#current-structure').val();
      var state = self.$el.find('#current-state').val();
      var period = self.$el.find('#current-period').val();

      var filter = [];

      if (company && company != 9999999) {
        var ResCompany = _.filter(self.ResCompany, function(item) {
          return item.id == company;
        });
        filter.push({
          title: 'Empresa',
          value: ResCompany[0].name,
        });
      }

      if (store && store != 9999999) {
        var ResStore = _.filter(self.ResStore, function(item) {
          return item.id == store;
        });

        filter.push({
          title: 'Sucursal',
          value: ResStore[0].name,
        });
      }

      if (employee && employee != 9999999) {
        var HrEmployee = _.filter(self.HrEmployee, function(item) {
          return item.id == employee;
        });

        filter.push({
          title: 'Funcionario',
          value: HrEmployee[0].name,
        });

      }
      if (structure && structure != 9999999) {
        var HrPayrollStructure = _.filter(self.HrPayrollStructure, function(item) {
          return item.id == structure;
        })
        filter.push({
          title: 'Estructura Salarial',
          value: HrPayrollStructure[0].name,
        });
      }

      if (state && state != 9999999) {
        filter.push({
          title: 'Estado',
          value: $('#current-state option:selected').text(),
        });
      }

      if (period && period != 9999999) {
        var AccountPeriod = _.filter(self.AccountPeriod, function(item) {
          return item.id == period;
        })
        filter.push({
          title: 'Período',
          value: AccountPeriod[0].name,
        });
      }

      return filter;
    },
  });
}
